import { Network, routes, Chain, ChainContext, TokenId, Signer, ChainAddress, AccountAddress } from '@wormhole-foundation/sdk-connect';
import { Ntt } from '@wormhole-foundation/sdk-definitions-ntt';
import { EvmNtt } from '@wormhole-foundation/sdk-evm-ntt';
import { EvmChains, EvmUnsignedTransaction } from '@wormhole-foundation/sdk-evm';
import { NttRoute } from '@wormhole-foundation/sdk-route-ntt';
import { TransactionRequest } from 'ethers';

type Op = NttRoute.Options;
type Tp = routes.TransferParams<Op>;
type Vr = routes.ValidationResult<Op>;
type Vp = NttRoute.ValidatedParams;
type QR = routes.QuoteResult<Op, Vp>;
type Q = routes.Quote<Op, Vp>;
type R = NttRoute.AutomaticTransferReceipt;
type Contracts = Ntt.Contracts & {
    wrappedMToken: string;
};
declare class M0AutomaticRoute<N extends Network> extends routes.AutomaticRoute<N, Op, Vp, R> implements routes.StaticRouteMethods<typeof M0AutomaticRoute> {
    static NATIVE_GAS_DROPOFF_SUPPORTED: boolean;
    static EVM_CONTRACTS: Contracts;
    static meta: {
        name: string;
        provider: string;
    };
    static supportedNetworks(): Network[];
    static supportedChains(network: Network): Chain[];
    static getContracts(chain: Chain): Contracts;
    static supportedSourceTokens(fromChain: ChainContext<Network>): Promise<TokenId[]>;
    static supportedDestinationTokens<N extends Network>(token: TokenId, fromChain: ChainContext<N>, toChain: ChainContext<N>): Promise<TokenId[]>;
    static isProtocolSupported<N extends Network>(chain: ChainContext<N>): boolean;
    getDefaultOptions(): Op;
    isAvailable(request: routes.RouteTransferRequest<N>): Promise<boolean>;
    validate(request: routes.RouteTransferRequest<N>, params: Tp): Promise<Vr>;
    quote(request: routes.RouteTransferRequest<N>, params: Vp): Promise<QR>;
    initiate(request: routes.RouteTransferRequest<N>, signer: Signer, quote: Q, to: ChainAddress): Promise<R>;
    /**
     * Modified from EvmNtt `transfer` function to call `transferMLikeToken` instead
     * https://github.com/wormhole-foundation/native-token-transfers/blob/main/evm/ts/src/ntt.ts#L461
     */
    transferMLike<N extends Network, C extends EvmChains>(ntt: EvmNtt<N, C>, sender: AccountAddress<C>, amount: bigint, destination: ChainAddress, sourceToken: string, destinationToken: string, options: Ntt.TransferOptions): AsyncGenerator<EvmUnsignedTransaction<N, C>>;
    createUnsignedTx<N extends Network, C extends EvmChains>(ntt: EvmNtt<N, C>, txReq: TransactionRequest, description: string, parallelizable?: boolean): EvmUnsignedTransaction<N, C>;
    track(receipt: R, timeout?: number): AsyncGenerator<R, void, unknown>;
}

export { M0AutomaticRoute };
